#pragma large
#pragma debug
#pragma registerbank(3)
#pragma pagelength(80) pagewidth(110)
/***********************************************************************
************************************************************************
*                                                                      *
*  C A N   D R I V E R                                                 *
*                                                                      *
************************************************************************
************************************************************************
*                                                                      *
*  Funktion:                                                           *
*                                                                      *
*     Diese Modul enthlt den eigentlichen CAN-Task. Der CAN-Task      *
*     bernimmt die gesammte Steuerung der CAN-Treiber Software.       *
*     Er reagiert auf Befehle von der Applikation (Signal und Daten    *
*     vom Modul CANAPIF) und auf Interrupts vom CAN-Controller.        *
*     Gestartet wird der CAN-Task vom Modul CANAPIF.                   *
*                                                                      *
*                                                                      *
*  Filename           :   PCANDRV.C51                                  *
*                                                                      *
*  Modulename         :   PCANDRV                                      *
*                                                                      *
*  Zielsystem         :   Jedes 8051 System mit einem Philips          *
*                         SJA1000                                      *
*                                                                      *
*  Entwicklungssystem :   Keil C-51                                    *
*                                                                      *
************************************************************************
*  Versionen:                                                          *
*                                                                      *
*  0.1  26.10.1990  Th. Fischler : Erste Version                       *
*  1.0   7.12.1990  Th. Fischler : Bus-off Handling eingebaut          *
*        21.1.1991  Th. Fischler : Send-Error Verhalten eingebaut      *
*        14.1.1991  Th. Fischler : Full-CAN eingebaut                  *
*        20.8.1991  Th. Fischler : Beim Basic-CAN wird der Error-INT   *
*                                  nicht mehr benutzt                  *
*  2.0  10.10.1991  Th. Fischler : Version fr RTX-51 V4.0             *
*  2.1  24.8.1992   Th. Fischler : Workaround fr Signal Mehrfach-     *
*                                  verwendungsproblem                  *
*                                                                      *
*                                                                      *
************************************************************************
*    Copyright 1991 .. 2001 METTLER  &  FUCHS  AG,  CH-8953 Dietikon   *
************************************************************************/

/*---------------------------------------------------------------------*
 *     D E F I N I T I O N E N                                         *
 *---------------------------------------------------------------------*/

/* Test ob der CAN-Controller angewhlt wurde */
#ifndef SJA1000
   #error "SJA1000 muss definiert werden"
#endif


#include "abbrev.h"
#include "candefs.h"


/* Befehle von der Applikation */
#define CAN_HW_INIT_ORDER       1
#define CAN_DEF_OBJ_ORDER       2
#define CAN_STOP_ORDER          3
#define CAN_START_ORDER         4
#define CAN_SEND_ORDER          5
#define CAN_REQUEST_ORDER       6
#define CAN_BIND_OBJ_ORDER      7
#define CAN_UNBIND_OBJ_ORDER    8
#define CAN_WRITE_ORDER         9
#define CAN_READ_W_IND_ORDER    10
#define CAN_READ_W_TID_ORDER    11
#define CAN_READ_ORDER          12
#define CAN_GET_STATUS_ORDER    13
#define CAN_ACTIVATE_WAIT       14   /* Spezial-Funktion zum Anzeigen,      */
				     /* das ein Task auf ein Objekt wartet  */


/* Function-Prototypes */
static void c_handle_receive_int (void);
static void c_handle_can_int (void);
static void c_get_back (byte ret_val);
static void c_handle_appli_signal (void);


/*---------------------------------------------------------------------*
 *     I M P O R T S                                                   *
 *---------------------------------------------------------------------*/


/* Verwendeter CAN-Interrupt (definiert in XCANCONF.A51) */
extern const unsigned char code CAN_INT_NBR;

/* RTX-51 Function-Calls */
#include <rtx51.h>

#include "pcanobja.h"
#include "pcancont.h"




/*---------------------------------------------------------------------*
 *     M O D U L G L O B A L E  V A R I A B L E N                      *
 *---------------------------------------------------------------------*/

   /* Die Applikation erteilt dem CAN-Task ber den gemeinsamen        */
   /* Speicherbereich C_ORDER_BUF Befehle. Ueber diesen gemeinsamen    */
   /* Speicherbereich gibt der CAN-Driver ebenfalls Status-Meldungen   */
   /* an die Applikation zurck.                                       */
   /* Der C_ORDER_BUF ist im Modul CANAPIF.A51 definiert.              */
   /* Die hier definierte Struktur wird an die selbe Speicherstelle    */
   /* gelegt.                                                          */
   /* ACHTUNG !! : Diese Struktur darf nicht einfach abgendert werden */
   /* sie muss mit der Definition im Modul CANAPIF.A51 bereinstimmen  */

   extern xdata struct {
		   byte order;
		   byte task_id;
		   byte param[6];
		} c_order_buf;


   /* Ist TRUE falls der CAN-Task gestoppt ist (keine Objekte senden   */
   /* oder empfangen kann) und FALSE falls der CAN-Task luft.         */
   static byte can_stopped;

   /* Zhlt die Anzahl aufgetretener Overrun-Interrupts vom            */
   /* Philips SJA1000 CAN-Controller (fr interne Testzwecke)           */
   static unsigned int c_overrun_counter;

   /* Workaround Signal-Problem                                        */
   /* Enthlt die identifikation aller Tasks an die ein Objekt         */
   /* gebunden ist und an die tatschlich mit CAN_WAIT auf das Objekt  */
   /* warten                                                           */
   static struct {
	       byte task_id;
	       byte task_waits;
	    } signal_ok[MAX_BIND_OBJ];


#pragma eject
/*---------------------------------------------------------------------*
 *     L O K A L E   P R O Z E D U R E N                               *
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------*
 *     C _ H A N D L E _ R E C E I V E _ I N T
 *---------------------------------------------------------------------*
 *  Funktion:
 *
 *     Steuert den Empfang eines Objektes nach einem Receive-Interrupt
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     --
 *
 *---------------------------------------------------------------------*/

static void c_handle_receive_int (void)
{
   unsigned int stat;
   byte         index, bind, i;

   stat = c_receive_obj ();
   if (stat <= 255) {
      /* Applikation muss benachrichtigt werden */
      index = stat;
      /* testen ob das empfangene Objekt an einen Task gebunden ist */
      bind = c_get_bind (index);
      if (bind != NO_BIND) {
	 /* Objekt ist an einen Task gebunden */
	 /* Testen ob Task bereit ist */
	 for (i=0; i<=MAX_BIND_OBJ-1; i++) {
	    if ((signal_ok[i].task_id == bind)
	    && (signal_ok[i].task_waits == TRUE)) {
	       os_send_signal (bind);
	       return;
	    }
	 }
      }
      else {
	 /* Objekt an keinen Task gebunden, an Mailbox senden          */
	 /* Die Applikation liest danach mit dem Index die Objektdaten */
	 if (os_send_message (7,index,0) == -1) {
	    /* Objekt kann von der Applikation nicht empfangen werden */
	    c_set_read (index);
	 }
      }
   }
   else {
      ;
      /* Ein Objekt wurde empfangen, die Applikation muss aber nicht */
      /* benachrichtigt werden.                                      */
      /* Dieser Fall wurde nur fr interne Testzwecke eingefgt      */
   }
}



/*---------------------------------------------------------------------*
 *     C _ H A N D L E _ C A N _ I N T
 *---------------------------------------------------------------------*
 *  Funktion:
 *
 *     Abhandlung der Aktionen die beim Auftreten eines CAN-Interrupts
 *     ausgefhrt werden mssen.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     --
 *
 *---------------------------------------------------------------------*/

static void c_handle_can_int (void)
{
   /* Die Informationsdarstellung ist beim Basic- und Full-CAN        */
   /* Controller nach einem Interrupt sehr unterschiedlich.           */
   /* Daher wird die Auswertung der Interrupt-Informationen fr       */
   /* die beiden Controller getrennt implementiert                    */


   can_interrupt_type int_r; /* Interrupt-Bits im CAN-Controller */

   /* Interrupt Register lesen, um den Grund des Interrupts zu          */
   /* eruieren und zum Rcksetzen des Interrupts                                                */
   int_r.bytes = c_get_int_reg ();

   /* Alle mglichen Interrupts testen, es knnen mehrere                       */
   /* gleichzeitig auftreten                                                                                    */
   if (int_r.bits.RI) {
      /* Receive-Interrupt */
      c_handle_receive_int();
   }

   if (int_r.bits.DOI) {
      /* Overrun-Interrupt */
      c_overrun_counter++;
      c_clr_overrun ();
   }
}



#pragma eject
/*---------------------------------------------------------------------*
 *     C _ G E T _ B A C K
 *---------------------------------------------------------------------*
 *  Funktion:
 *
 *     Rckgabe des Statuswert RET_VAL an die Applikation, starten
 *     der Applikation mit einem Signal
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     ret_val  :  Wert der an die Applikation zurckgegeben werden soll
 *
 *---------------------------------------------------------------------*/

static void c_get_back (byte ret_val)
{
   c_order_buf.param[0] = ret_val;
   os_send_signal (c_order_buf.task_id);
}




/*---------------------------------------------------------------------*
 *     C _ H A N D L E _ A P P L I _ S I G N A L
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Ausfhrung aller Applikationsbefehle
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     --
 *
 *---------------------------------------------------------------------*/

static void c_handle_appli_signal (void)
{
   byte temp, temp2;
   byte done, i;
   unsigned int w_temp;
   /* Hilfsvariable zum separaten ansprechen von High- und Low-Byte  */
   /* in einem Word                                                  */
   union {
      unsigned int tw;
      byte tb[2];
   } qq;

   /* Union um den den Pointer als einzelne Bytes ansprechen zu knnen   */
   union {
      struct
      {  byte offs[2];
      } short_ptr;
      unsigned int xdata *i_ptr;
      byte xdata *c_ptr;
   } po;

   can_control_type cont_r;

   switch (c_order_buf.order) {
      case CAN_HW_INIT_ORDER:
	 c_stop ();
	 can_stopped = TRUE;
	 done = c_hw_init (c_order_buf.param[0], /* bus_timing_0 */
			   c_order_buf.param[1], /* bus_timing_1 */
			   c_order_buf.param[2], /* out control */
			   c_order_buf.param[3]); /* clock devider */
	 if (done == FALSE) {
	    c_get_back (C_CAN_FAILURE);
	    return;
	 }

	 /************ NUR Receive- und Overrun Interrupt enablen ********/
	 /* und SYNC-Mode auf 0 einstellen                               */
	 cont_r.bytes = 0;
	 cont_r.bits.RIE = 1;
	 cont_r.bits.OIE = 1;
	 cont_r.bits.S = 0x00; /* set sync immer auf 0 */

	 c_init_cont_reg(cont_r.bytes);


	 c_init_obj_buf ();
	 for (i=0; i<=MAX_BIND_OBJ-1; i++) {
	    signal_ok[i].task_id = NO_BIND;
	    signal_ok[i].task_waits = FALSE;
	 }
	 c_overrun_counter = 0;
      break;

      case CAN_DEF_OBJ_ORDER:
	 if (can_stopped == FALSE) {
	    c_get_back (C_NOT_STOPPED);
	    return;
	 }
	 qq.tb[0] = c_order_buf.param[0];  /* HIGH Ident */
	 qq.tb[1] = c_order_buf.param[1];  /* LOW Ident */
	 temp = c_define_obj (
                  qq.tw,                   /* Ident */
                  c_order_buf.param[2],    /* data length */
		  c_order_buf.param[3]     /* obj type */
                             );
	 if (temp != C_OK) {
	    c_get_back (temp);
	    return;
	 }
      break;

      case CAN_STOP_ORDER:
	 c_stop ();
	 can_stopped = TRUE;
      break;

      case CAN_START_ORDER:
	 /* Akkzeptanz-Maske des Controllers setzen */
	 temp = c_get_acc_code ();
	 temp2 = c_get_acc_mask ();
	 c_init_accept_reg (temp, temp2);

	 /* Alle Objekte im Objekt-Buffer als gelesen markieren   */
	 /* zum Erreichen eines definierten Anfangszustandes      */
	 c_set_all_read ();
	 c_start ();
	 can_stopped = FALSE;
	 if (c_get_status() == bus_off) {
			/* wenn der CAN-Controller bereits nach dem Starten im */
			/* Bus-Off Zustand ist, ist etwas schief gelaufen          */
			c_get_back (C_CAN_FAILURE);
			return;
		 }
      break;

      case CAN_SEND_ORDER:
	 if (c_get_status() == bus_off) {
	    c_get_back (C_BUS_OFF);
	    return;
	 }
	 /* Pointer auf das Datenfeld initialisieren */
	 po.short_ptr.offs[0] = c_order_buf.param[0];
	 po.short_ptr.offs[1] = c_order_buf.param[1];
	 temp = c_get_index_from_ident (*po.i_ptr);
	 if (temp == 0) {
	    c_get_back (C_OBJ_ERROR);
	    return;
	 }
	 temp2 = c_get_obj_typ (temp);
	 if ((temp2 != D_SEND) && (temp2 != D_SEND_R_REC)) {
	    c_get_back (C_OBJ_ERROR);
	    return;
	 }
		 done = c_transmit_obj (temp, po.c_ptr+2);
	 if (done==FALSE) {
	    c_get_back (C_SEND_ERROR);
	    return;
	 }
      break;

      case CAN_REQUEST_ORDER:
	 if (c_get_status() == bus_off) {
	    c_get_back (C_BUS_OFF);
	    return;
	 }
	 qq.tb[0] = c_order_buf.param[0];
	 qq.tb[1] = c_order_buf.param[1];
	 temp = c_get_index_from_ident (qq.tw);
	 if (temp == 0) {
	    c_get_back (C_OBJ_ERROR);
	    return;
	 }
	 temp2 = c_get_obj_typ (temp);
	 if (temp2 != D_REC_R_SEND) {
	    c_get_back (C_OBJ_ERROR);
	    return;
	 }
	 done = c_send_remote (temp);
	 if (done==FALSE) {
	    c_get_back (C_SEND_ERROR);
	    return;
	 }
      break;

      case CAN_BIND_OBJ_ORDER:
	 qq.tb[0] = c_order_buf.param[0];
	 qq.tb[1] = c_order_buf.param[1];
	 temp = c_get_index_from_ident (qq.tw);
	 if (temp == 0) {
	    c_get_back (C_OBJ_ERROR);
	    return;
	 }
	 temp2 = c_get_bind (temp);
	 done = c_set_bind (temp, c_order_buf.param[2]);
	 for (i=0; i<=MAX_BIND_OBJ-1; i++) {
	    if ((signal_ok[i].task_id == NO_BIND)
	    || (signal_ok[i].task_id == c_order_buf.param[2])) {
	       signal_ok[i].task_id = c_order_buf.param[2];
	       signal_ok[i].task_waits = FALSE;
	       break;
	    }
	 }

	 if (done == FALSE) {
	    c_get_back (C_MEM_FULL);
	    return;
	 }
	 if (temp2 != NO_BIND) {
	    /* Objekt war schon an Task gebunden */
	    c_get_back (C_OBJ_REBIND);
	    return;
	 }
      break;

      case CAN_UNBIND_OBJ_ORDER:
	 qq.tb[0] = c_order_buf.param[0];
	 qq.tb[1] = c_order_buf.param[1];
	 temp = c_get_index_from_ident (qq.tw);
	 if (temp == 0) {
	    c_get_back (C_OBJ_ERROR);
	    return;
	 }
	 temp2 = c_get_bind(temp);
	 c_clear_bind (temp);
	 for (i=0; i<=MAX_BIND_OBJ-1; i++) {
	    if (signal_ok[i].task_id == temp2) {
	       signal_ok[i].task_id = NO_BIND;
	       signal_ok[i].task_waits = FALSE;
	       break;
	    }
	 }

      break;

      case CAN_WRITE_ORDER:
	 /* Pointer auf das Datenfeld initialisieren */
	 po.short_ptr.offs[0] = c_order_buf.param[0];
	 po.short_ptr.offs[1] = c_order_buf.param[1];
	 temp = c_get_index_from_ident (*po.i_ptr);
	 if (temp == 0) {
	    c_get_back (C_OBJ_ERROR);
	    return;
	 }
	 temp2 = c_get_obj_typ (temp);
	 if ((temp2 != D_SEND) && (temp2 != D_SEND_R_REC)) {
	    c_get_back (C_OBJ_ERROR);
	    return;
	 }
	 c_write_data (temp, po.c_ptr+2);
      break;

      case CAN_READ_W_IND_ORDER:
	 /* Pointer auf das Datenfeld initialisieren */
	 po.short_ptr.offs[0] = c_order_buf.param[0];
	 po.short_ptr.offs[1] = c_order_buf.param[1];
	 w_temp = c_get_ident (c_order_buf.param[2]);
	 if (w_temp == 0xffff) {
	    if (c_get_status() == bus_off) {
	       c_get_back (C_BUS_OFF);
	    }
	    else {
	       c_get_back (C_CAN_FAILURE);
	    }
	    return;
	 }
	 *po.i_ptr = w_temp;
	 c_read_data (c_order_buf.param[2], po.c_ptr+2);
      break;

      case CAN_READ_W_TID_ORDER:
	 /* Pointer auf das Datenfeld initialisieren */
	 po.short_ptr.offs[0] = c_order_buf.param[0];
	 po.short_ptr.offs[1] = c_order_buf.param[1];
	 temp = c_get_rec_bind (c_order_buf.param[2]);
	 if (temp == 0) {
	    c_get_back (C_CAN_FAILURE);
	    return;
	 }
	 *po.i_ptr = c_get_ident (temp);
	 c_read_data (temp, po.c_ptr+2);
	 for (i=0; i<=MAX_BIND_OBJ-1; i++) {
	    if (signal_ok[i].task_id == c_order_buf.param[2]) {
	       signal_ok[i].task_waits = FALSE;
	       break;
	    }
	 }
      break;

      case CAN_READ_ORDER:
	 qq.tb[0] = c_order_buf.param[2];
	 qq.tb[1] = c_order_buf.param[3];
	 temp = c_get_index_from_ident (qq.tw);
	 if (temp == 0) {
	    c_get_back (C_OBJ_ERROR);
	    return;
	 }
	 /* Pointer auf das Datenfeld initialisieren */
	 po.short_ptr.offs[0] = c_order_buf.param[0];
	 po.short_ptr.offs[1] = c_order_buf.param[1];
	 *po.i_ptr = c_get_ident (temp);
	 c_read_data (temp, po.c_ptr+2);
      break;

      case CAN_GET_STATUS_ORDER:
	 if (c_get_status() == error_active) {
	    c_get_back (C_ERR_ACTIVE);
	 }
	 else if (c_get_status() == error_passive) {
	    c_get_back (C_ERR_PASSIVE);
	 }
	 else if (c_get_status() == bus_off) {
	    c_get_back (C_BUS_OFF);
	 }
	 else {
	    c_get_back (C_CAN_FAILURE);
	 }
	 return;
      break;

      case CAN_ACTIVATE_WAIT:
	 for (i=0; i<=MAX_BIND_OBJ-1; i++) {
	    if (signal_ok[i].task_id == (c_order_buf.task_id)) {
	       signal_ok[i].task_waits = TRUE;
	       break;
	    }
	 }
	 if (c_get_rec_bind (c_order_buf.task_id) != 0) {
	    os_send_signal (c_order_buf.task_id);
	 }
	 return;
      break;

      default:
	 c_get_back (C_CAN_FAILURE);
	 return;
      break;
   }
   c_get_back (C_OK);
}




#pragma eject
/*---------------------------------------------------------------------*
 *     C A N - T A S K                                                 *
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------*
 *     C A N _ C A N _ T A S K _ M A I N
 *---------------------------------------------------------------------*
 *  Funktion:
 *
 *     Hauptschleife des CAN-Task.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     --
 *
 *---------------------------------------------------------------------*/

void can_can_task_main (void) _task_ CAN_TASK_ID _priority_ 3
{
   byte i;

   /* Grundinitialisierung beim Starten des CAN-Task          */
   /* CAN-Controller auf anhalten, bis zur Initialisierung    */
   /* durch die Applikation drfen keine Interrupts auftreten */
   can_stopped = TRUE;
   c_first_init ();
   c_init_obj_buf ();
   c_overrun_counter = 0;
   for (i=0; i<=MAX_BIND_OBJ-1; i++) {
      signal_ok[i].task_id = NO_BIND;
      signal_ok[i].task_waits = FALSE;
   }

   /* CAN-Interrupt zuordnen */
   os_attach_interrupt (CAN_INT_NBR);

   /* Hauptschleife des Task */
   for (;;) {
      /* Auf Signal von der Applikation oder Interrupt vom  */
      /* CAN-Controller warten                              */
      if (os_wait (K_INT+K_SIG, 0xff, 0) == INT_EVENT) {
	 /* CAN-Controller Interrupt abhandeln */
	 c_handle_can_int ();
      }
      else {
	 /* Signal von der Applikation abhandeln */
	 c_handle_appli_signal ();
      }
   }
}


/*---------------------------------------------------------------------*
 *         Ende Modul PCANDRV                                                                                       *
 *---------------------------------------------------------------------*/
