#pragma large
#pragma debug
#pragma registerbank(3)
#pragma pagelength(80) pagewidth(110)
/***********************************************************************
************************************************************************
*                                                                      *
*  80592 - C A N   C O N T R O L L E R                                 *
*                                                                      *
************************************************************************
************************************************************************
*                                                                      *
*  Funktion:                                                           *
*                                                                      *
*     Direkte HW-Steuerung der CAN-Schnittstelle des Philips 80592.    *
*     Ausfhrung aller Low-Level Funktionen. Dieses Modul ist das      *
*     einzige, dass direkt in die HW-Register des CAN-Controllers      *
*     schreiben darf.                                                  *
*                                                                      *
*                                                                      *
*  Filename           :  CCONT592.C51                                  *
*                                                                      *
*  Modulename         :  CCONT592                                      *
*                                                                      *
*  Zielsystem         :  Philips 80592 Prozessor                       *
*                                                                      *
*  Entwicklungssystem :  Keil C-51                                     *
*                                                                      *
************************************************************************
*  Versionen:                                                          *
*                                                                      *
*  1.0  17.10.1991  Th. Fischler : Erste Version                       *
*                                                                      *
************************************************************************
*    Copyright 1991 .. 2001 METTLER  &  FUCHS  AG,  CH-8953 Dietikon   *
************************************************************************/

/*---------------------------------------------------------------------*
 *     D E F I N I T I O N E N                                         *
 *---------------------------------------------------------------------*/

#include "abbrev.h"
#include "ccont592.h"

/* Maximale Anzahl der Sende-Versuche bei Bus-Problemen */
/* von CCONF592.A51                                     */
extern const unsigned char code NBR_OF_SEND_TRIALS;


/* Definition der einzelnen Bits im STATUS-REGISTER    */
/* des CAN-Controllers                                 */
union can_status_typ {
   struct {
      byte rec_buffer_stat  : 1;
      byte data_overrun     : 1;
      byte trans_buf_access : 1;
      byte trans_complete   : 1;
      byte rec_status       : 1;
      byte trans_status     : 1;
      byte err_status       : 1;
      byte bus_status       : 1;
   } status_bits;
   byte status_byte;
};


/* Definition der internen Adressen der CAN-Register (82C200 Register) */
#define CONTROL   0
#define COMMAND   1
#define STATUS    2
#define INTERRUPT 3
#define ACC_CODE  4
#define ACC_MASK  5
#define BUS_T_0   6
#define BUS_T_1   7
#define OUT_CONTR 8
#define TRANSMIT  10
#define RECEIVE   20


/*---------------------------------------------------------------------*
 *     I M P O R T S                                                   *
 *---------------------------------------------------------------------*/


/* RTX-51 Function-Calls */
#include <rtx51.h>

#include "canutil.h"


/*---------------------------------------------------------------------*
 *     M O D U L G L O B A L E  V A R I A B L E N                      *
 *---------------------------------------------------------------------*/

/* Definition der speziellen 80592 CAN-Steuer-Register */

sfr CANADR = 0xdb;
sfr CANDAT = 0xda;
sfr CANCON = 0xd9;
sfr CANSTA = 0xd8;


#pragma eject
/*---------------------------------------------------------------------*
 *     L O K A L E  P R O Z E D U R E N                                *
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------*
 *     S E T _ I N T E R N A L _ R E G
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Setzt das interne CAN-Register 'reg' auf den Wert 'value'
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *     reg      : Interne Adresse des zu beschreibenden Registers
 *     value    : Wert der geschrieben werden soll
 *
 *---------------------------------------------------------------------*/

static void set_internal_reg (byte reg, byte value)
{
   CANADR = reg;
   CANDAT = value;
}


/*---------------------------------------------------------------------*
 *     G E T _ I N T E R N A L _ R E G
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Liest das interne CAN-Register 'reg'.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *     reg           : Interne Adresse des zu lesenden Registers
 *
 *     Rckgabewert  : Aktueller Wert des Register 'reg'
 *
 *---------------------------------------------------------------------*/

static byte get_internal_reg (byte reg)
{
   CANADR = reg;
   return CANDAT;
}

                 
/*---------------------------------------------------------------------*
 *     C A N _ R E S E T _ R E Q U E S T
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Setzt den CAN-Controller in den Reset-Zustand
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *     --
 *
 *---------------------------------------------------------------------*/

static void can_reset_request (void)
{
   byte cont;

   cont = get_internal_reg (CONTROL);
   cont = cont | 0x01;
   set_internal_reg (CONTROL, cont);
}



#pragma eject
/*---------------------------------------------------------------------*
 *     E X P O R T I E R T E  P R O Z E D U R E N                      *
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------*
 *     C _ F I R S T _ I N I T
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Diese Routine sollte vor der ersten
 *     initialisierung des Controllers aufgerufen werden.
 *     (Sie ist vorallem aus zur Sicherstellung der Kompatibilitt zur
 *     Basic-CAN Software vorhanden).
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *     --
 *---------------------------------------------------------------------*/
void c_first_init (void)
{
   can_reset_request();
}


/*---------------------------------------------------------------------*
 *     C _ H W _ I N I T
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Grundinitialisierung des 80592 CAN-Controllers.
 *     Der CAN-Controller wird fr diese Operation angehalten.
 *     Er muss danach mit C_START wieder neu gestartet werden.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     bus_t0_reg   : Bus-Timing Register 0 des 80592
 *     bus_t1_reg   : Bus-Timing Register 1 des 80592
 *     out_c_reg    : Output-Control Register des 80592
 *
 *     Returnwert   : TRUE   -> OK
 *                    FALSE  -> Fehler beim Init
 *---------------------------------------------------------------------*/

byte c_hw_init (byte bus_t0_reg,
                byte bus_t1_reg,
                byte out_c_reg)
{
   byte cont;

   can_reset_request();
   /* test-mode ausschalten */
   cont = get_internal_reg (CONTROL);
   cont = cont & 0x7f;;
   set_internal_reg (CONTROL, cont);
   set_internal_reg (BUS_T_0, bus_t0_reg);
   set_internal_reg (BUS_T_1, bus_t1_reg);
   set_internal_reg (OUT_CONTR, out_c_reg);
   return TRUE;
}



/*---------------------------------------------------------------------*
 *     C _ I N I T _ C O N T _ R E G
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Initialisierung des CONTROL-REGISTERS des 80592
 *     (Festlegung der Interrupts und des Synch-Modes).
 *     Der CAN-Controller wird fr diese Operation angehalten, er muss
 *     danach mit C_START wieder neu gestartet werden.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     cont_reg    :  Inhalt des 80592 Control-Registers
 *
 *---------------------------------------------------------------------*/

void c_init_cont_reg (byte cont_reg)
{
   /* Reset-Request Bit einschalten und sicherstellen dass Test-Mode */
   /* ausgeschaltet bleibt                                           */
   set_internal_reg (CONTROL, (cont_reg | 0x01) & 0x7f);
}



/*---------------------------------------------------------------------*
 *     C _ I N I T _ A C C E P T _ R E G
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Acceptance-Code und Acceptance-Mask Register initialisieren.
 *     Der CAN-Controller wird fr diese Operation angehalten, er muss
 *     danach mit C_START wieder neu gestartet werden.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     acc_code    :  Inhalt des Acceptanc-Code Registers
 *     acc_mask    :  Inhalt des Acceptance-Mask Registers
 *
 *---------------------------------------------------------------------*/

void c_init_accept_reg (byte acc_code,
                        byte acc_mask)
{
   can_reset_request();
   set_internal_reg (ACC_CODE, acc_code);
   set_internal_reg (ACC_MASK, acc_mask);
}



/*---------------------------------------------------------------------*
 *     C _ S T A R T
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Starten des CAN-Controllers nach Init oder nach C_STOP.
 *     Nach C_START knnen Objekte empfangen und gesendet werden.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     --
 *
 *---------------------------------------------------------------------*/

void c_start (void)
{
   /* Das Reset request Bit kann durch einen externen Hardware Reset */
   /* ausgeschaltet bleiben, darum solange ausschalten bis der Reset */
   /* Zustand wirklich freigegeben ist                               */
   can_reset_request();
   set_internal_reg (CONTROL, get_internal_reg(CONTROL) & 0x0fe );
   while (get_internal_reg(CONTROL) & 0x01) {
      set_internal_reg (CONTROL, get_internal_reg(CONTROL) & 0x0fe );
      os_wait (K_TMO, 2, 0);
   }
}



/*---------------------------------------------------------------------*
 *     C _ S T O P
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Temporres Anhalten des CAN-Controllers. Es kann mit C_START
 *     weitergefahren werden. Solange C_STOP aktiv ist, knnen keine
 *     Objekte empfangen oder gesendet werden.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     --
 *
 *---------------------------------------------------------------------*/

void c_stop (void)
{
   can_reset_request();
}



/*---------------------------------------------------------------------*
 *     C _ G E T _ R E C _ I D
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Abfrage des Identifiers des gerade empfangenen Objektes.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     Returnwert     : Identifier des gerade empfangenen Objektes
 *
 *---------------------------------------------------------------------*/

unsigned int c_get_rec_id (void)
{
   /* Hilfsvariable zum separaten ansprechen von High- und Low-Byte  */
   /* in einem Word                                                  */
   register union {
               unsigned int tw;
               byte tb[2];
            } temp;

   temp.tb[0] = get_internal_reg(RECEIVE) >> 5;
   temp.tb[1] = (get_internal_reg(RECEIVE+1) >> 5) + (get_internal_reg(RECEIVE) << 3);
   return temp.tw;
}



/*---------------------------------------------------------------------*
 *     C _ R E C _ R E M O T E
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Gibt zurck ob das eben empfangene Objekt ein Remote-Frame ist.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     Returnwert   : TRUE  -> Remote-Frame
 *                    FALSE -> Data-Frame
 *
 *---------------------------------------------------------------------*/

byte c_rec_remote (void)
{
   if (get_internal_reg(RECEIVE+1) & 0x10) {
      return TRUE;
   }
   return FALSE;
}



/*---------------------------------------------------------------------*
 *     C _ G E T _ R E C _ L E N
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Gibt die Datenlnge des eben empfangenen Objektes zurck.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     Returnwert  :  Anzahl Datenbytes die empfangen wurden (0..8)
 *
 *---------------------------------------------------------------------*/

byte c_get_rec_len (void)
{
   return (get_internal_reg(RECEIVE+1) & 0x0f);
}



/*---------------------------------------------------------------------*
 *     C _ R E A D _ R E C _ D A T A
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Lesen der Daten des eben empfangenen Objektes. Die Daten werden
 *     in den XDATA-Bereich, dessen Anfang mit dem Parameter BUF_PTR
 *     bezeichnet ist, kopiert.
 *     Es werden soviele Bytes kopiert wie tatschlich empfangen wurden
 *     (kann mit C_GET_REC_LEN gelesen werden).
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     arr       : Array im XDATA-Bereiche in den die
 *                 eben gerade empfangenen Daten kopiert werden knnen.
 *
 *---------------------------------------------------------------------*/

void c_read_rec_data (byte xdata *arr)
{
   byte len;
   byte i;

   len = c_get_rec_len();
   /* Startadresse mit Auto-Inkrement laden */
   CANADR = (RECEIVE+2) | 0x20;
   for (i=0; i<len; i++) {
      *arr = CANDAT;
      arr++;
   }
}



/*---------------------------------------------------------------------*
 *     C _ R E L _ R E C _ R E G
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Nach dem Empfang eines Objektes kann der Empfangsbuffer von
 *     der CPU gelesen werden. Whrend dieser Zeit kann der
 *     CAN-Controller keine neue Daten in den Buffer schreiben.
 *     Nach dem Lesen der Daten muss der Buffer wieder fr den CAN-
 *     Controller freigegeben werden.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     --
 *
 *---------------------------------------------------------------------*/

void c_rel_rec_reg (void)
{
   CANCON = (get_internal_reg(COMMAND) & 0xe0) | 0x04;
}



/*---------------------------------------------------------------------*
 *     C _ S E N D _ O B J
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Uebertragung eines Objektes zum CAN-Controller zum Senden.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     identifier     :  Identifier des zu sendenden Objektes
 *     remote         :  TRUE  -> sende ein Remote-Frame
 *                       FALSE -> sende ein Data-Frame
 *     length         :  Anzahl Datenbytes die gesendet werden mssen
 *     arr            :  Array mit den Daten die gesendet werden sollen
 *
 *---------------------------------------------------------------------*/

byte c_send_obj (unsigned int identifier,
                 byte remote,
                 byte length,
                 byte xdata *arr)
{
   register union {
               unsigned int tw;
               byte tb[2];
            } temp;

   byte nbr_of_trials;  /* Anzahl Sendeversuche bis Abbruch */
   byte i;
   byte len;
   byte xdata *ptr;

   nbr_of_trials = 0;
   /* Falls der Buffer noch besetzt ist, warten */
   while (!(CANSTA & 0x04)) {
	  nbr_of_trials++;
	  if (nbr_of_trials > 10) {
         os_wait (K_TMO, 2, 0);
         if (nbr_of_trials > NBR_OF_SEND_TRIALS+10) {
			/* Bus wird nicht frei, vermutlich ist kein anderer Teilnehmer */
			/* am Bus, abbrechen										   */
			return FALSE;
		 }
      }
   }

   /* Identifier in den Buffer schreiben */
   temp.tw = identifier;
   set_internal_reg (TRANSMIT, (temp.tb[0] << 5) + (temp.tb[1] >> 3));
   if (remote) {
      /* rtr-bit setzen */
      set_internal_reg (TRANSMIT+1, (temp.tb[1] << 5) | 0x10);
   }
   else {
      set_internal_reg (TRANSMIT+1, (temp.tb[1] << 5)+length);
      /* Data-Frame */
      /* Daten in den Buffer schreiben */
      len = length;
      ptr = arr;
      CANADR = (TRANSMIT+2) | 0x20;
      for (i=0; i<len; i++) {
         CANDAT = *ptr;
         ptr++;
      }
   }
   /* Transmission Request Bit setzen */
   CANCON = (get_internal_reg(COMMAND) & 0xe0) | 0x01;
   return TRUE;
}



/*---------------------------------------------------------------------*
 *     C _ C L R _ O V E R R U N
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Lschen des Overrun-Status im 82C200.
 *
 *
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     --
 *
 *---------------------------------------------------------------------*/

void c_clr_overrun (void)
{
   /* Overrun-Status zurcksetzen und Receive-Buffer releasen */
   CANCON = (get_internal_reg(COMMAND) & 0xe0) | 0x0c;
   /* fr die erste Controller Version kurz Controller reseten */
   can_reset_request();
   set_internal_reg (CONTROL, get_internal_reg(CONTROL) & 0x0fe );
   while (get_internal_reg(CONTROL) & 0x01) {
      set_internal_reg (CONTROL, get_internal_reg(CONTROL) & 0x0fe );
      os_wait (K_TMO, 2, 0);
   }
}



/*---------------------------------------------------------------------*
 *     C _ G E T _ I N T _ R E G
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Diese Funktion liest den Inhalt des Interrupt-Registers
 *     des 82C200.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     Returnwert  :  Inhalt des 82C200 Interrupt-Registers
 *
 *---------------------------------------------------------------------*/

byte c_get_int_reg (void)
{
   return CANCON;
}



/*---------------------------------------------------------------------*
 *     C _ G E T _ S T A T U S
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Gibt den aktuellen Bus-Zustand des 82C200 zurck
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     Returnwert  :  Aktueller 82C200 Bus-Zustand
 *
 *---------------------------------------------------------------------*/

enum can_bus_stat_t c_get_status (void)
{
   union can_status_typ stat_r;   /* Status-Bits im CAN-Controller    */

   stat_r.status_byte = CANSTA;
   if (stat_r.status_bits.bus_status) {
      return bus_off;
   }
   if (stat_r.status_bits.err_status) {
      return error_passive;
   }
   return error_active;
}
