#pragma large
#pragma debug
#pragma registerbank(3)
#pragma pagelength(80) pagewidth(110)
/***********************************************************************
************************************************************************
*                                                                      *
*  F U L L - C A N   C O N T R O L L E R                               *
*                                                                      *
************************************************************************
************************************************************************
*                                                                      *
*  Funktion:                                                           *
*                                                                      *
*     Direkte HW-Steuerung des Siemens C515C interne Full-CAN          *
*     Controllers.                                                     *
*     Ausfhrung aller Low-Level Funktionen. Dieses Modul ist das      *
*     einzige, dass direkt in die HW-Register des CAN-Controllers      *
*     schreiben darf.                                                  *
*                                                                      *
*                                                                      *
*  Filename           :  IANCONT.C51                                   *
*                                                                      *
*  Modulename         :  IANCONT                                       *
*                                                                      *
*  Zielsystem         :  jedes System mit einem Siemens C515           *
*                        interne CAN-Controller                        *
*                                                                      *
*  Entwicklungssystem :  Keil C-51                                     *
*                                                                      *
************************************************************************
*  Versionen:                                                          *
*                                                                      *
*  1.0  23.05.1997  K. Birsen : Erste Version                          *
*  1.1  11.06.1997  K. Birsen : Extended Identification                *
*                               nicht mehr auf 0xFFFF begrenzt         *
*  5.3  22.7.1997   K. Birsen : 15. Object implementiert.              *
*                                                                      *
************************************************************************
*    Copyright 1991 .. 2001 METTLER  &  FUCHS  AG,  CH-8953 Dietikon   *
************************************************************************/

/*---------------------------------------------------------------------*
 *     D E F I N I T I O N E N                                         *
 *---------------------------------------------------------------------*/

sfr   SYSCONA = 0xB1;

#include "abbrev.h"
#include "icancont.h"
#include "candefs.h"

/* Maximale Anzahl der verschiedenen Versuche */
/* von ICACNCONF.A51                          */
extern const unsigned int code NBR_OF_SEND_TRIALS;

#define CONTROL_REG_0_1_BIT_SET          2    /* 10 */
#define CONTROL_REG_0_1_BIT_RESET        1    /* 01 */
#define TRANSMIT                         1
#define RECEIVE                          0

/* verschiedene Ofsets des Message-Registers */
#define MCR0_OFFSET                0
#define MCR1_OFFSET                1
#define UAR0_OFFSET                2
#define MCFG_OFFSET                6
#define MSG_DATA_OFFSET            7

#define BTR0_OFFSET                0x04
#define BTR1_OFFSET                0x05
#define ARBITR_OFFSET              0x02
#define UMLM_OFFSET                0x0C


/* Definition der einzelnen Bits im CONTROL-REGISTER    */
/* des CAN-Controllers                                  */
    typedef struct {
        byte INIT              : 1;        /* rw 1 */
        byte IE                : 1;        /* rw 0 */
        byte SIE               : 1;        /* rw 0 */
        byte EIE               : 1;        /* rw 0 */
        byte reserve_1         : 1;        /* r  0 */
        byte reserve_2         : 1;        /* r  0 */
        byte CCE               : 1;        /* rw 0 */
        byte TEST              : 1;        /* rw 0 */
    } CR_typ;

/* Definition der einzelnen Bits im STATUS-REGISTER    */
/* des CAN-Controllers                                 */
    typedef struct {
        byte LEC               : 3;        /* rw x */
        byte TXOK              : 1;        /* rw x */
        byte RXOK              : 1;        /* rw x */
        byte reserve           : 1;        /* r  x */
        byte EWRN              : 1;        /* r  x */
        byte BOFF              : 1;        /* r  x */
    } SR_type;

    /* Registers der Messages  */
    typedef struct {
        byte INTPND            : 2;        /* rw x */
        byte RXIE              : 2;        /* rw x */
        byte TXIE              : 2;        /* rw x */
        byte MSGVAL            : 2;        /* rw x */
    } MCR0_type;
    typedef struct {
        byte NEWDAT            : 2;        /* rw x */
        byte MSGLST_or_CPUUPD  : 2;        /* rw x */
        byte TXRQ              : 2;        /* rw x */
        byte RMTPND            : 2;        /* rw x */
    } MCR1_type;
    typedef struct {
        byte reserve           : 2;        /* r  0 */
        byte XTD               : 1;        /* rw x */
        byte DIR               : 1;        /* rw x */
        byte DLC               : 4;        /* rw x */
    } MCFG_type;

    typedef struct {
        byte BRP               : 6;        /* rw x */
        byte SLW               : 2;        /* rw x */
    } BTR0_type;
    typedef struct {
        byte TSEG1             : 4;        /* rw x */
        byte TSEG2             : 3;        /* rw x */
        byte reseve            : 1;        /* r  x */
    } BTR1_type;

    /* Message type */
    typedef struct {
/* 00H */        MCR0_type     MCR0;
/* 01H */        MCR1_type     MCR1;
/* 02H */        unsigned long arbitration;        /*arbitration_0,arbitration_1,arbitration_2,arbitration_3 */
/* 06H */        MCFG_type     MCFG;
/* 07H */        byte          datas[8];
/* 0FH */        byte			 reseve;
    } message_type;


/*---------------------------------------------------------------------*
 *     I M P O R T S                                                   *
 *---------------------------------------------------------------------*/

#include "canutil.h"

/*---------------------------------------------------------------------*
 *     M O D U L G L O B A L E  V A R I A B L E N                      *
 *---------------------------------------------------------------------*/
  /* Definition der Siemens C515C HW-Register, diese werden an die     */
  /* Speicherstelle des Symbols FCAN_REG im Assembler-Modul            */
  /* ICANCONF.A51 alloziert. (absolute Variable)                       */

   extern xdata volatile union
    {
        struct{
/* 00H */   CR_typ           CR;            /* 0x01 */
/* 01H */   SR_type          SR;            /* undefined */
/* 02H */   byte             IR;            /* undefined */
/* 03H */   byte             reserve;
/* 04H */   BTR0_type        BTR0;          /* unchanged */
/* 05H */   BTR1_type        BTR1;          /* unchanged */
/* 06H */   unsigned int     GMS0;          /* unchanged */
/* 08H */   unsigned long    UGML0;         /* unchanged */
/* 0CH */   unsigned long    UMLM0;         /* unchanged */

/* 10H */   message_type     messages[15];  /* unchanged */
        } r;
        byte                 mem[256];
    } fcan_reg;



/* Um diese Registers zum ndern control.CCE muss 1 sein */
/*             BTR0      */
/*             BTR1      */



byte tmp_last_data[12];



#pragma eject
/*---------------------------------------------------------------------*
 *     L O K A L E  P R O Z E D U R E N                                *
 *---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*
 *     C A N _ R E S E T _ R E Q U E S T
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Setzt den CAN-Controller in den Reset-Zustand
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *     --
 *
 *---------------------------------------------------------------------*/

static byte can_reset_request (void)
{
    unsigned int nr_of_trials;

    /* setze CCE(cpu hat Schreibrecht auf bit timing-Reg.) */
    fcan_reg.r.CR.INIT = 1;
    fcan_reg.r.CR.CCE = 1;

    /*  Warte bis RESET PIN = low (return FALSE nach timeout) */
    nr_of_trials=0;
    while (fcan_reg.r.CR.INIT != 1){
        nr_of_trials++; 
        if (nr_of_trials > NBR_OF_SEND_TRIALS){
            return FALSE;
        }
    }
    return TRUE;
}



/*---------------------------------------------------------------------*
 *     C A N _ W A I T _ U N T I L _ S E N D E D
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Warte bis ein Object mit Erfolgt gesendt
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     Returnwert     :  TRUE  --> Data mit Erfolg gesendet
 *                       FALSE --> Data nicht gesendet (time out )
 *
 *---------------------------------------------------------------------*/

static byte can_wait_until_sended (void)
{
    unsigned int nr_of_trials=0;

    /*   warte bis TXOK = 1 oder timeout */
    do{
        nr_of_trials++; 
        if (nr_of_trials > NBR_OF_SEND_TRIALS){
            return FALSE;
        }
    } while (fcan_reg.r.SR.TXOK == 0 );

    return TRUE;
}

/*---------------------------------------------------------------------*
 *     C A N _ S E T _A R B I T R A T I O N
 *---------------------------------------------------------------------*
 *  Funktion:
 *        Lade standart oder extended arbitration eines Objectes.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *    arbitr_offset:  Interne Start-Adresse der Objektarbitration
 *    ident        :  Object Arbitration
 *    standart     :  TRUE wenn standart sonst(extendet) FALSE
 *
 *---------------------------------------------------------------------*/

static can_set_arbitrations (byte arbitr_offset,
                             unsigned long ident,
                             byte standart
                             )
{
    union {
        unsigned long  arbitration;
        byte           mem[4];
    }tmp_arbitration;

    tmp_arbitration.arbitration = ident;

    if (standart){
        tmp_arbitration.arbitration = tmp_arbitration.arbitration << 21;
    }
    else{
        tmp_arbitration.arbitration = tmp_arbitration.arbitration << 3;
    }

    fcan_reg.mem[arbitr_offset + 0] = tmp_arbitration.mem[0];
    fcan_reg.mem[arbitr_offset + 1] = tmp_arbitration.mem[1];
    fcan_reg.mem[arbitr_offset + 2] = tmp_arbitration.mem[2];
    fcan_reg.mem[arbitr_offset + 3] = tmp_arbitration.mem[3];
}

/*---------------------------------------------------------------------*
 *     C A N _ G E T _A R B I T R A T I O N
 *---------------------------------------------------------------------*
 *  Funktion:
 *          Momentan nicht bentzt
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *    arbitr_offset:  Interne Start-Adresse der Objektarbitration
 *
 *    Returnwert   :  Object Arbitration
 *
 *---------------------------------------------------------------------*/


static unsigned long can_get_arbitrations (byte arbitr_offset)

{
    union {
        unsigned long  arbitration;
        byte           mem[4];
    }tmp_arbitration;

    byte obj_inx;


    tmp_arbitration.mem[0] = fcan_reg.mem[arbitr_offset + 0];
    tmp_arbitration.mem[1] = fcan_reg.mem[arbitr_offset + 1];
    tmp_arbitration.mem[2] = fcan_reg.mem[arbitr_offset + 2];
    tmp_arbitration.mem[3] = fcan_reg.mem[arbitr_offset + 3];


    obj_inx = arbitr_offset - ARBITR_OFFSET - CONTR_BUF_START_ADR;
    obj_inx = (obj_inx >> 4);
    
    if (fcan_reg.r.messages[obj_inx].MCFG.XTD == 0){ /* standart */
        tmp_arbitration.arbitration = tmp_arbitration.arbitration >> 21;
    }
    else{
        tmp_arbitration.arbitration = tmp_arbitration.arbitration >> 3;
    }

    return (tmp_arbitration.arbitration);
}



#pragma eject
/*---------------------------------------------------------------------*
 *     E X P O R T I E R T E  P R O Z E D U R E N                      *
 *---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*
 *     C _ H W _ I N I T
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Grundinitialisierung des C515C CAN-Controllers.
 *     Fr diese Operation wird der CAN-Controller angehalten.
 *     Er muss danach mit C_START wieder neu gestartet werden.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     bus_t0_reg       : Bus-Timing Register 0 des C515C CAN Controller
 *     bus_t1_reg       : Bus-Timing Register 1 des C515C CAN Controller
 *     can Bus freq = XTAL / [(BRP+1)*(3+TSEG1+TSEG2)]
 *
 *
 *     Returnwert   : TRUE   -> OK
 *                    FALSE  -> Fehler beim Reset
 *---------------------------------------------------------------------*/

byte c_hw_init (byte bus_t0_reg,
                byte bus_t1_reg,
                byte dummy_1,
                byte dummy_2)
{
    int                inx;

    /* XMAP0 = 0  (damit CAN-Kontroller Zugriff mglich wird) */
    SYSCONA &= 0xFE; 

    /* only to evoid warning */
    inx = dummy_1;
    inx = dummy_2;

    if (can_reset_request () ){
        fcan_reg.mem[BTR0_OFFSET] = bus_t0_reg;
        fcan_reg.mem[BTR1_OFFSET] = bus_t1_reg;

        fcan_reg.r.CR.EIE = 0; /* EIE */
        fcan_reg.r.CR.SIE = 0; /* SIE */
        fcan_reg.r.CR.IE = 1; /* IE */

        fcan_reg.r.SR.LEC = 0;
        fcan_reg.r.SR.TXOK = 0;
        fcan_reg.r.SR.RXOK = 0;

        /* initialisiere alle 15 Objekte  */
        /********************************************************************/
        for (inx = 0; inx <= 14; inx++){
				fcan_reg.r.messages[inx].MCR0.TXIE = CONTROL_REG_0_1_BIT_RESET;
				fcan_reg.r.messages[inx].MCR0.RXIE = CONTROL_REG_0_1_BIT_RESET;
				fcan_reg.r.messages[inx].MCR0.INTPND = CONTROL_REG_0_1_BIT_RESET;
				fcan_reg.r.messages[inx].MCR1.RMTPND = CONTROL_REG_0_1_BIT_RESET;
				fcan_reg.r.messages[inx].MCR1.TXRQ = CONTROL_REG_0_1_BIT_RESET;
				fcan_reg.r.messages[inx].MCR1.MSGLST_or_CPUUPD = CONTROL_REG_0_1_BIT_SET;
				fcan_reg.r.messages[inx].arbitration = 0xFFFFFFFF;
				fcan_reg.r.messages[inx].MCR1.NEWDAT = CONTROL_REG_0_1_BIT_RESET;
				fcan_reg.r.messages[inx].MCFG.DIR = RECEIVE;
				fcan_reg.r.messages[inx].MCFG.DLC = 0;
				fcan_reg.r.messages[inx].MCR0.MSGVAL = CONTROL_REG_0_1_BIT_RESET;
				fcan_reg.r.messages[inx].MCFG.XTD = 0; /* standart */

				fcan_reg.r.messages[inx].datas[0]= inx;
				fcan_reg.r.messages[inx].datas[1]= inx;
				fcan_reg.r.messages[inx].datas[2]= inx;
				fcan_reg.r.messages[inx].datas[3]= inx;
				fcan_reg.r.messages[inx].datas[4]= inx;
				fcan_reg.r.messages[inx].datas[5]= inx;
				fcan_reg.r.messages[inx].datas[6]= inx;
				fcan_reg.r.messages[inx].datas[7]= inx;

        }
        /* make filtering of all individual message arbitrations activ */
        fcan_reg.r.GMS0 = 0xFFFF;
        fcan_reg.r.UGML0 = 0xFFFFFFFF;
        fcan_reg.r.UMLM0 = 0xFFFFFFFF;

        return TRUE;
    }
    else{
        return FALSE;
    }
}


/*---------------------------------------------------------------------*
 *     C _ I N I T _ O B J
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Definiert an der internen CAN-Controller Adresse BUF_ADR ein
 *     neues CAN-Objekt. Der CAN-Controller wird fr diese Operation
 *     angehalten, er muss danach mit C_START wieder neu gestartet
 *     werden.
 *
 *
 *---------------------------------------------------------------------*
 *
 *  Parameter:
 *        }
 *     ident       :  Identifier des zu definierenden Objektes
 *                    (0 .. 7EFH)
 *     data_len    :  Anzahl Datenbytes des Objektes (0 .. 8)
 *     obj_typ     :  Genaue Beschreibung der Objektart :
 *                    D_REC (0)         :  Receive data frame
 *                    D_SEND (1)        :  Send data frame
 *                    D_SEND_R_REC  (2) :  send data after remote recieving 
 *                    D_REC_R_SEND  (3) :  send remote frame
 *    obj_adr      :  Interne Start-Adresse des Objektes
 *    standart     :  TRUE wenn standart sonst(extendet) FALSE
 *
 *---------------------------------------------------------------------*/

void c_init_obj (unsigned long ident,
                byte         data_len,
                byte         obj_typ,
                byte         obj_adr,
                byte         standart)

{
    byte obj_inx;

    c_stop();

    obj_inx = obj_adr - CONTR_BUF_START_ADR;
    obj_inx = (obj_inx >> 4);

    if (standart == TRUE){
        fcan_reg.r.messages[obj_inx].MCFG.XTD = 0; /* standart */
    }
    else{
        fcan_reg.r.messages[obj_inx].MCFG.XTD = 1; /* extended */
    }
    fcan_reg.r.messages[obj_inx].MCFG.DLC = data_len;

    can_set_arbitrations (obj_adr + ARBITR_OFFSET, ident, standart);

    if (obj_adr == CONTR_BUF_END_ADR){
        can_set_arbitrations (UMLM_OFFSET , ident, standart);
    }

    fcan_reg.r.messages[obj_inx].MCR0.MSGVAL = CONTROL_REG_0_1_BIT_SET;
    fcan_reg.r.messages[obj_inx].MCR0.TXIE   = CONTROL_REG_0_1_BIT_RESET;
    fcan_reg.r.messages[obj_inx].MCR0.INTPND = CONTROL_REG_0_1_BIT_RESET;

    fcan_reg.r.messages[obj_inx].MCR1.RMTPND = CONTROL_REG_0_1_BIT_RESET;
    fcan_reg.r.messages[obj_inx].MCR1.TXRQ   = CONTROL_REG_0_1_BIT_RESET;
    fcan_reg.r.messages[obj_inx].MCR1.MSGLST_or_CPUUPD = CONTROL_REG_0_1_BIT_RESET;
    fcan_reg.r.messages[obj_inx].MCR1.NEWDAT = CONTROL_REG_0_1_BIT_RESET;

    /* wenn nur Data senden */
    if (obj_typ == D_SEND){
        fcan_reg.r.messages[obj_inx].MCFG.DIR  = TRANSMIT;
        fcan_reg.r.messages[obj_inx].MCR0.RXIE = CONTROL_REG_0_1_BIT_RESET;
    } 

    /* wenn remote senden und Antwort-Data empfangen */ 
    if (obj_typ == D_REC_R_SEND  ){
        fcan_reg.r.messages[obj_inx].MCFG.DIR  = RECEIVE;
        fcan_reg.r.messages[obj_inx].MCR0.RXIE = CONTROL_REG_0_1_BIT_SET;
    } 

    /* wenn nur Data empfangen */
    if (obj_typ == D_REC){
        fcan_reg.r.messages[obj_inx].MCFG.DIR  = RECEIVE;
        fcan_reg.r.messages[obj_inx].MCR0.RXIE = CONTROL_REG_0_1_BIT_SET;
    } 

    /* wenn Remote empfangen und Answer-Daten senden */
    if (obj_typ == D_SEND_R_REC){
        fcan_reg.r.messages[obj_inx].MCFG.DIR  = TRANSMIT;
        fcan_reg.r.messages[obj_inx].MCR0.RXIE = CONTROL_REG_0_1_BIT_RESET;
    } 
}


/*---------------------------------------------------------------------*
 *     C _ S T A R T
 *---------------------------------------------------------------------*
 *  Funktion: 0k
 *     Starten des CAN-Controllers nach Init
 *     Nach C_START knnen Objekte empfangen und gesendet werden.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     --
 *
 *---------------------------------------------------------------------*/

void c_start (void)
{

    /* setze CCE(cpu hat Schreibrecht auf timing-Reg.) */
    fcan_reg.r.CR.INIT = 0;
    fcan_reg.r.CR.CCE = 0;

    fcan_reg.r.CR.EIE = 1; /* EIE */
    fcan_reg.r.CR.SIE = 0; /* SIE */
    fcan_reg.r.CR.IE = 1; /* IE */

    fcan_reg.r.SR.LEC = 0;
    fcan_reg.r.SR.TXOK = 0;
    fcan_reg.r.SR.RXOK = 0;
}


/*---------------------------------------------------------------------*
 *     C _ S T O P
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Temporres Anhalten des CAN-Controllers. Es kann mit C_START
 *     weitergefahren werden. Solange C_STOP aktiv ist, knnen keine
 *     Objekte empfangen oder gesendet werden.
 *
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     --
 *
 *---------------------------------------------------------------------*/

void c_stop (void)
{
    /* setze CCE=1 (cpu hat Schreibrecht auf timing-Register  */
    fcan_reg.r.CR.CCE = 1;
    fcan_reg.r.CR.INIT = 1;

    fcan_reg.r.CR.EIE = 0; /* EIE */
    fcan_reg.r.CR.SIE = 0; /* SIE */
    fcan_reg.r.CR.IE  = 0; /* IE */

}


/*---------------------------------------------------------------------*
 *     C _ R E A D _ O B J
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Lesen der Daten eines Objektes. Die Daten werden
 *     in den XDATA-Bereich, dessen Anfang mit dem Parameter BUF_PTR
 *     bezeichnet ist, kopiert. Es werden soviele Bytes kopiert wie bei
 *     C_INIT_OBJ definiert.
 *
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *    obj_adr      : Interne Start-Adresse des Objektes
 *
 *     arr         : Array im XDATA-Bereiche in den die
 *                   Daten kopiert werden knnen.
 *
 *---------------------------------------------------------------------*/

void c_read_obj (byte obj_adr,
                 byte xdata *arr)
{

    byte dat_len;
    unsigned int nr_of_trials;
    byte obj_inx;


    obj_inx = obj_adr - CONTR_BUF_START_ADR;
    obj_inx = (obj_inx >> 4);

    dat_len = fcan_reg.r.messages[obj_inx].MCFG.DLC;

    if (obj_adr == CONTR_BUF_END_ADR){
        c_fast_copy( tmp_last_data, arr - 4, dat_len + 4 );
    }
    else{

        nr_of_trials = 0;
        do{
            fcan_reg.r.messages[obj_inx].MCR1.NEWDAT = CONTROL_REG_0_1_BIT_RESET;
            fcan_reg.r.messages[obj_inx].MCR1.MSGLST_or_CPUUPD = CONTROL_REG_0_1_BIT_RESET;

            nr_of_trials++; 

            c_fast_copy( &fcan_reg.r.messages[obj_inx].datas[0], arr, dat_len );

        } while ((fcan_reg.r.messages[obj_inx].MCR1.NEWDAT == CONTROL_REG_0_1_BIT_SET)
            && (nr_of_trials < NBR_OF_SEND_TRIALS));
    }
}


/*---------------------------------------------------------------------*
 *     C _ S E N D _ O B J
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Uebertragung eines Objektes zum CAN-Controller zum Senden.
 *     Es werden soviele Byte gesendet, wie bei C_INIT_OBJ definiert
 *     (nur wenn nicht Remote).
 *
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *    obj_adr      :  Interne Start-Adresse des Objektes
 *     remote         :  TRUE  -> sende ein Remote-Frame
 *                       FALSE -> sende ein Data-Frame
 *     arr            :  Array im XDATA-Bereich mit den Daten die gesendet
 *                       werden sollen
 *
 *     Returnwert     :  TRUE  -> Senden erfolgreich
 *                       FALSE -> Senden nicht erfolgreich
 *
 *---------------------------------------------------------------------*/

byte c_send_obj (byte obj_adr,
                 byte remote,
                 byte xdata *arr)
{

    byte obj_inx;
    byte dat_len;
    byte ret_val;

    obj_inx = obj_adr - CONTR_BUF_START_ADR;
    obj_inx = (obj_inx >> 4);

    fcan_reg.r.SR.TXOK = 0;

    if (!remote){
        /* setze MSGLST_or_CPUUPD whrend Dataspeichern */
        fcan_reg.r.messages[obj_inx].MCR1.MSGLST_or_CPUUPD = CONTROL_REG_0_1_BIT_SET;

        dat_len = fcan_reg.r.messages[obj_inx].MCFG.DLC;
        c_fast_copy(arr, &fcan_reg.r.messages[obj_inx].datas[0], dat_len );

        /* Ende Dataspeichern : reset MSGLST_or_CPUUPD */
        fcan_reg.r.messages[obj_inx].MCR1.MSGLST_or_CPUUPD = CONTROL_REG_0_1_BIT_RESET;

        fcan_reg.r.messages[obj_inx].MCR1.NEWDAT = CONTROL_REG_0_1_BIT_SET;
    }
    else{
        fcan_reg.r.messages[obj_inx].MCR1.RMTPND = CONTROL_REG_0_1_BIT_RESET;
    }
    fcan_reg.r.messages[obj_inx].MCR1.TXRQ = CONTROL_REG_0_1_BIT_SET;

    if (can_wait_until_sended () ){
        ret_val = TRUE;
    }
    else{
        ret_val = FALSE;
    }

    fcan_reg.r.messages[obj_inx].MCR1.TXRQ = CONTROL_REG_0_1_BIT_RESET;

    return ret_val;
}


/*---------------------------------------------------------------------*
 *     C _ W R I T E _ O B J
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Speichern eines Objektes im CAN-Controller, die Daten werden
 *     nicht gesendet sondern stehen zum Beispiel fr eine Anfrage
 *     durch ein Remote-Frame zur Verfgung. Es werden soviele Byte
 *     abgespeichert, wie bei C_INIT_OBJ definiert.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *    obj_adr      :  Interne Start-Adresse des Objektes
 *     arr            :  Array im XDATA-Bereich mit den Daten die
 *                       geschrieben werden sollen
 *
 *     Returnwert     :  immer TRUE
 *
 *---------------------------------------------------------------------*/

byte c_write_obj (byte obj_adr,
                  byte xdata *arr)
{
    byte dat_len;
    byte obj_inx;

    obj_inx = obj_adr - CONTR_BUF_START_ADR;
    obj_inx = (obj_inx >> 4);

    /* setze MSGLST_or_CPUUPD whrend Dataspeichern */
    fcan_reg.r.messages[obj_inx].MCR1.MSGLST_or_CPUUPD = CONTROL_REG_0_1_BIT_SET;

    dat_len = fcan_reg.r.messages[obj_inx]. MCFG.DLC;
    c_fast_copy(arr, &fcan_reg.r.messages[obj_inx].datas[0], dat_len );

    /* Ende Dataspeichern : zurcksetze MSGLST_or_CPUUPD */
    fcan_reg.r.messages[obj_inx].MCR1.MSGLST_or_CPUUPD = CONTROL_REG_0_1_BIT_RESET;
    fcan_reg.r.messages[obj_inx].MCR1.NEWDAT = CONTROL_REG_0_1_BIT_SET;

    return TRUE;
}


/*---------------------------------------------------------------------*
 *     C _ G E T _ I N T _ T Y P
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Diese Funktion gibt den Typ des Aufgetretenen Interrupts zurck.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     Returnwert  :
 *                no_int               :
 *                error_int            :     Status register Inhalt gendert
 *                                           wenn (SIE = 1)
 *                obj_int              :     object number+2 is in
 *                                           interrupt_register
 *                obj_15_int           :     15. object interrupt_register
 *
 *---------------------------------------------------------------------*/

enum interrupt_t c_get_int_typ (void)
{

    if (fcan_reg.r.IR == 0x00){
        return no_int;
    }

    /* Status Register Inhalt gendert (wenn SIE = 1) */
    else if (fcan_reg.r.IR == 0x01){
        return error_int;
    }

    /* Message Object 1 bis 14 */
    else if ((fcan_reg.r.IR >= 0x03)
        &&   (fcan_reg.r.IR <= 0x10) ) {
        return obj_int;
    }

    /* wennf Objekt 15 Interrupt pending */
    if (fcan_reg.r.IR == 2){ 
        return obj_int;
    }

}


/*---------------------------------------------------------------------*
 *     C _ G E T _ O B J _ I N T _ A D R
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Gibt die Startadresse des Objektes(1..15) bei dem ein Objekt-Interrupt
 *     aufgetreten ist zurck oder 0 wenn kein intr. pending.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     Returnwert  :
 *                return interrupt_register wenn interrupt_register = 0 oder 1
 *                return F0 wenn interrupt_register = 2 (15 Message )
 *                Startaddress des Objektes bei dem ein Interrupt aufgetreten ist
 *
 *---------------------------------------------------------------------*/

byte c_get_obj_int_adr (void)
{
    byte adr;

    /* wenn Objekt 1..14 Interrupt pending */
    if(   (fcan_reg.r.IR >= 0x03) && (fcan_reg.r.IR <= 0x10)
        ){
        adr = fcan_reg.r.IR - 3;
        adr = adr << 4;
        return (adr + CONTR_BUF_START_ADR);
    }

    /* wennf Objekt 15 Interrupt pending */
    if (fcan_reg.r.IR == 2){ 
        return CONTR_BUF_END_ADR;
    }

    /* Keine Interrupt pending oder staus register is erneuert */
    if (   (fcan_reg.r.IR == 0) /* keine intr. pending */
        || (fcan_reg.r.IR == 1) /* status reg ist erneuert */
        ){
        return fcan_reg.r.IR;
    }
}

/*---------------------------------------------------------------------*
 *     C _ R E C E I P T _ O B J _ I N T
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Diese Funktion setzt einen Objekt-Interrupt des Objektes mit der
 *     internen Startadresse obj_inx zurck.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     obj_adr        :  Interne Adresse des Objektes dessen Interrupt
 *                       werden soll.
 *---------------------------------------------------------------------*/

void c_receipt_obj_int (byte obj_adr)
{
    byte obj_inx;

    union{
        unsigned long arbitration;
        byte mem[4];
    }tmp_arbitration;

    obj_inx = obj_adr - CONTR_BUF_START_ADR;
    obj_inx = (obj_inx >> 4);

    fcan_reg.r.messages[obj_inx].MCR0.INTPND = CONTROL_REG_0_1_BIT_RESET;

    if (obj_adr == CONTR_BUF_END_ADR){
        tmp_arbitration.arbitration = can_get_arbitrations (obj_adr + ARBITR_OFFSET);

        tmp_last_data[0] = tmp_arbitration.mem[0];
        tmp_last_data[1] = tmp_arbitration.mem[1];
        tmp_last_data[2] = tmp_arbitration.mem[2];
        tmp_last_data[3] = tmp_arbitration.mem[3];

        c_fast_copy( &fcan_reg.r.messages[obj_inx].datas[0], &tmp_last_data[4], 8 );
    }

    fcan_reg.r.messages[obj_inx].MCR1.NEWDAT = CONTROL_REG_0_1_BIT_RESET;
    fcan_reg.r.messages[obj_inx].MCR1.RMTPND = CONTROL_REG_0_1_BIT_RESET;
}


/*---------------------------------------------------------------------*
 *     C _ R E C E I P T _ E R R O R _ I N T
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Diese Funktion quittiert einen Error-Interrupt
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *    --
 *---------------------------------------------------------------------*/

void c_receipt_error_int (void)
{
   SR_type dummy_SR;

    /* dummy Lesen vom Status-Register um  */
    dummy_SR = fcan_reg.r.SR;

    fcan_reg.r.SR.LEC = 0;
}


/*---------------------------------------------------------------------*
 *     C _ G E T _ S T A T U S
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Gibt den aktuellen Bus-Zustand des C515C CAN Controller.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     Returnwert  :  Bus-Zustand des C515C CAN Controller
 *
 *---------------------------------------------------------------------*/

enum can_bus_stat_t c_get_status (void)
{
    if (fcan_reg.r.SR.BOFF == 1){
        return bus_off;
    }
    if (fcan_reg.r.SR.LEC == 0){
        return error_active;
    }
    return error_passive;

}
