
#pragma large
#pragma debug
#pragma registerbank(3)
#pragma pagelength(80) pagewidth(110)
/***********************************************************************
************************************************************************
*                                                                      *
*  C A N   O B J E C T  A D M I N I S T R A T I O N                    *
*                                                                      *
************************************************************************
************************************************************************
*                                                                      *
*  Funktion:                                                           *
*                                                                      *
*     Dieses Modul bernimmt die Verwaltung von allen vom Benutzer     *
*     definierten Kommunikationsobjekten. Es regelt den Empfang und    *
*     das Senden der Kommunikationsobjekte.                            *
*                                                                      *
*                                                                      *
*                                                                      *
*  Filename           :   ICANOBJA.C51                                 *
*                                                                      *
*  Modulename         :   ICANOBJA                                     *
*                                                                      *
*  Zielsystem         :   Jedes C515C System                           *
*                                                                      *
*  Entwicklungssystem :   Keil C-51                                    *
*                                                                      *
************************************************************************
*  Versionen:                                                          *
*                                                                      *
*  0.1  12.06.1997  K. Birsen    : Erste Version                       *
*  1.1  11.06.1997  K. Birsen : Extended Identification                *
*                               nicht mehr auf 0xFFFF begrenzt         *
*  5.3  22.7.1997   K. Birsen : 15. Object implementiert.              *
*                                                                      *
************************************************************************
*    Copyright 1991 .. 2001 METTLER  &  FUCHS  AG,  CH-8953 Dietikon   *
************************************************************************/

/*---------------------------------------------------------------------*
 *     D E F I N I T I O N E N                                         *
 *---------------------------------------------------------------------*/

#include "abbrev.h"
#include "icanobja.h"
#include "candefs.h"

/* Maximale Anzahl Objekte im Objektbuffer */
#define MAX_OBJECTS 15


/* Maximale Lnge der Daten in einem Objekt */
#define MAX_OBJECT_DATA_SIZE 8

/* Grsster Objektidentifier, der benutzt werden darf */
#define MAX_OBJECT_IDENT 0x7FF
#define MAX_OBJECT_IDENT_EXT 0x1FFFFFFF


/*---------------------------------------------------------------------*
 *     I M P O R T S                                                   *
 *---------------------------------------------------------------------*/

#include "icancont.h"
#include "canutil.h"


/*---------------------------------------------------------------------*
 *     M O D U L G L O B A L E  V A R I A B L E N                      *
 *---------------------------------------------------------------------*/

    /* Definition des Objektbuffers                    */
    /* Das Objekt mit dem Index 0 wird nicht verwendet !                */
    /* (Dummy-Element, zum Anzeigen der Nichtexistenz eines Objektes)   */
    static xdata struct {
                   unsigned long identifier;    /* CAN Objekt-ID     */
                   byte obj_typ;        /* Typ des Objekts    */
                   byte standart;       /* TRUE wenn standart */
                   byte data_length;    /* Anzahl Datenbytes    */
                   byte task_bind;      /* Task-ID oder FFH    */
                   byte obj_read;       /* Objekt gelesen ?    */
                   byte contr_adr;      /* interne Start-    */
            } can_obj_buf[MAX_OBJECTS+1];    /* Adresse des Objekts  */

   /* Anzahl definierte Objekte                                        */
   static xdata byte nbr_of_obj;

   /* Enthlt die Indexe aller Objekte die an einen bestimmten Task    */
   /* gebunden sind (deren Feld OBJ_BUF.TASK_BIND ungleich FFH ist).   */
   /* Der Wert 0 zeigt ein unbenutztes Feld an (enthlt keinen Index). */
   /* Mit Hilfe dieses Arrays kann schnell ein zu einem bestimmten     */
   /* Task gehrendes Objekt gefunden werden.                          */
   static xdata byte bind_index_list[MAX_BIND_OBJ];


#pragma eject
/*---------------------------------------------------------------------*
 *       L O K A L E     P R O Z E D U R E N                               *
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------*
*      C _ G E T _ I N D E X _ F R O M _ A D R
*---------------------------------------------------------------------*
*  Funktion:
*      Testet ob das Objekt mit der Controller internen Adresse C_ADR
*      im Objekt-Buffer definiert ist und gibt den entsprechenden Index
*      innerhalb von CAN_OBJ_BUF zurck.
*
*---------------------------------------------------------------------*
*  Parameter:
*
*      c_adr   :    Controller interne Start-Adresse des Objektes
*
*      Returnwert   :    0        -->   Das Objekt existiert nicht
*                        1 .. 15  -->   Index des Objektes
*
*---------------------------------------------------------------------*/

static byte c_get_index_from_adr (byte c_adr)
{
  byte i, temp;

  /* wenn 15. object */
  if (c_adr == CONTR_BUF_END_ADR ){
     return(MAX_OBJECTS);
  }

  temp = nbr_of_obj;
  for (i=1; i<=temp; i++) {
     if (can_obj_buf[i].contr_adr == c_adr) return i;
  }
  return 0;

}


#pragma eject
/*---------------------------------------------------------------------*
 *     E X P O R T I E R T E  P R O Z E D U R E N                      *
 *---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*
 *     Initialisieren und Definieren von Objekten                      *
 *---------------------------------------------------------------------*/

/*---------------------------------------------------------------------*
 *     C _ I N I T _ O B J _  B U F
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Neu Initialisieren des Objektbuffers, Lschen aller vorhandenen
 *     Objekte
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     --
 *
 *---------------------------------------------------------------------*/

void c_init_obj_buf (void)
{
   byte i;

   nbr_of_obj = 0;
   for (i=0; i<=MAX_BIND_OBJ-1; i++) bind_index_list[i] = 0;

   for (i=0; i<=MAX_OBJECTS; i++) can_obj_buf[i].identifier = 0;

}



/*---------------------------------------------------------------------*
 *     C _ D E F I N E _ O B J
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Definiert ein neues Objekt im Objekt-Buffer
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     identifier   : Identifikation des zu definierenden Objektes
 *     data_length  : Anzahl Datenbytes des zu definierenden
 *                    Objektes (0..8)
 *     obj_typ      : Genauere Beschreibung der Verwendungsart des
 *                    Objektes :
 *
 *                    D_REC        :   Objekt kann nur empfangen werden
 *                    D_SEND       :   Objekt kann nur gesendet werden
 *                    D_REC_R_SEND :   Objekt kann empfangen werden
 *                                     und es kann ein entsprechende
 *                                     Remote-Frame ausgesendet werden
 *                    D_SEND_R_REC :   Objekt kann gesendet werden und es
 *                                     kann ein entsprechendes Remote-Frame
 *                                     empfangen und beantwortet werden
 *
 *
 *     Returnwert   : OK, OBJ_ERROR, TOO_LONG, INVALID_TYPE, MEM_FULL
 *
 *---------------------------------------------------------------------*/

    byte c_define_obj (unsigned int identifier,
                       byte data_length,
                       byte obj_typ)

    {

          /* Test der Input-Werte */
          if (data_length > MAX_OBJECT_DATA_SIZE) return C_TOO_LONG;

          if (   (obj_typ != D_REC)
              && (obj_typ != D_SEND)
              && (obj_typ != D_REC_R_SEND)
              && (obj_typ != D_SEND_R_REC))
             return C_INVALID_TYPE;

          if (nbr_of_obj >= MAX_OBJECTS-1) return C_MEM_FULL;
          if (identifier > MAX_OBJECT_IDENT) return C_OBJ_ERROR;

          if (c_get_index_from_ident(identifier) == 0) {
             nbr_of_obj++;
             can_obj_buf[nbr_of_obj].identifier = identifier;
             can_obj_buf[nbr_of_obj].obj_typ = obj_typ;
             can_obj_buf[nbr_of_obj].standart = TRUE;
             can_obj_buf[nbr_of_obj].data_length = data_length;
             can_obj_buf[nbr_of_obj].task_bind = NO_BIND;
             can_obj_buf[nbr_of_obj].obj_read = TRUE;

             can_obj_buf[nbr_of_obj].contr_adr =
                 CONTR_BUF_START_ADR + (nbr_of_obj - 1) * MESSAGE_SIZE;

             c_init_obj (identifier,
                        data_length,
                        obj_typ,
                        can_obj_buf[nbr_of_obj].contr_adr,
                        TRUE); /* 1 for standart */
             return C_OK;
          }
          else {
             /* Objekt existiert schon */
             return C_OBJ_ERROR;
          }
   }

/*---------------------------------------------------------------------*
 *     C _ D E F I N E _ O B J _ E X T
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Definiert ein neues Objekt im Objekt-Buffer
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     identifier   : Identifikation des zu definierenden Objektes
 *     data_length  : Anzahl Datenbytes des zu definierenden
 *                    Objektes (0..8)
 *     obj_typ      : Genauere Beschreibung der Verwendungsart des
 *                    Objektes :
 *
 *                    D_REC        :   Objekt kann nur empfangen werden
 *                    D_SEND       :   Objekt kann nur gesendet werden
 *                    D_REC_R_SEND :   Objekt kann empfangen werden
 *                                     und es kann ein entsprechende
 *                                     Remote-Frame ausgesendet werden
 *                    D_SEND_R_REC :   Objekt kann gesendet werden und es
 *                                     kann ein entsprechendes Remote-Frame
 *                                     empfangen und beantwortet werden
 *
 *
 *     Returnwert   : OK, OBJ_ERROR, TOO_LONG, INVALID_TYPE, MEM_FULL
 *
 *---------------------------------------------------------------------*/

    byte c_define_obj_ext (unsigned long identifier,
                       byte data_length,
                       byte obj_typ)

    {

          /* Test der Input-Werte */
          if (data_length > MAX_OBJECT_DATA_SIZE) return C_TOO_LONG;

          if (   (obj_typ != D_REC)
              && (obj_typ != D_SEND)
              && (obj_typ != D_REC_R_SEND)
              && (obj_typ != D_SEND_R_REC))
             return C_INVALID_TYPE;

          if (nbr_of_obj >= MAX_OBJECTS-1) return C_MEM_FULL;
          if (identifier > MAX_OBJECT_IDENT_EXT) return C_OBJ_ERROR;

          if (c_get_index_from_ident(identifier) == 0) {
             nbr_of_obj++;
             can_obj_buf[nbr_of_obj].identifier = identifier;
             can_obj_buf[nbr_of_obj].obj_typ = obj_typ;
             can_obj_buf[nbr_of_obj].standart = FALSE;
             can_obj_buf[nbr_of_obj].data_length = data_length;
             can_obj_buf[nbr_of_obj].task_bind = NO_BIND;
             can_obj_buf[nbr_of_obj].obj_read = TRUE;

             can_obj_buf[nbr_of_obj].contr_adr =
                 CONTR_BUF_START_ADR + (nbr_of_obj - 1) * MESSAGE_SIZE;

             c_init_obj (identifier,
                        data_length,
                        obj_typ,
                        can_obj_buf[nbr_of_obj].contr_adr,
                        FALSE); /* 0 for extended */
             return C_OK;
          }
          else {
             /* Objekt existiert schon */
             return C_OBJ_ERROR;
          }
    }

/*---------------------------------------------------------------------*
 *     C _ D E F I N E _ L _ O B J
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Definiert 15. Objekt im Objekt-Buffer
 *       resultierende Akzeptanz-Maske
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     mask_15      : 15. Obj Mask
 *     data_length  : Anzahl Datenbytes des zu definierenden
 *                    Objektes (0..8)
 *
 *     Returnwert   : OK, OBJ_ERROR, TOO_LONG
 *
 *---------------------------------------------------------------------*/

    byte c_define_l_obj (unsigned long mask_15,
                       byte data_length
                       )

    {
          if (mask_15 > MAX_OBJECT_IDENT) return C_OBJ_ERROR;

          if (can_obj_buf[MAX_OBJECTS].identifier == 0) {
             can_obj_buf[MAX_OBJECTS].identifier = mask_15;
             can_obj_buf[MAX_OBJECTS].obj_typ = D_REC;
             can_obj_buf[MAX_OBJECTS].standart = TRUE;
             can_obj_buf[MAX_OBJECTS].data_length = data_length;
             can_obj_buf[MAX_OBJECTS].task_bind = NO_BIND;
             can_obj_buf[MAX_OBJECTS].obj_read = TRUE;

             can_obj_buf[MAX_OBJECTS].contr_adr =
                 CONTR_BUF_START_ADR + (MAX_OBJECTS -1 ) * MESSAGE_SIZE ;

             c_init_obj (mask_15,
                        data_length,
                        D_REC,
                        can_obj_buf[MAX_OBJECTS].contr_adr,
                        TRUE); /* 1 for standart */
             return C_OK;
          }
          else {
             /* Objekt existiert schon */
             return C_OBJ_ERROR;
          }
   }

/*---------------------------------------------------------------------*
 *     C _ D E F I N E _ L _ O B J _ E X T
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Definiert 15. Objekt im Objekt-Buffer
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     mask_15      : 15. Obj Mask
 *     data_length  : Anzahl Datenbytes des zu definierenden
 *                    Objektes (0..8)
 *
 *     Returnwert   : OK, OBJ_ERROR, TOO_LONG
 *
 *---------------------------------------------------------------------*/

    byte c_define_l_obj_ext (unsigned long mask_15,
                       byte data_length
                       )

    {

          if (mask_15 > MAX_OBJECT_IDENT_EXT) return C_OBJ_ERROR;

          if (can_obj_buf[MAX_OBJECTS].identifier == 0) {
             can_obj_buf[MAX_OBJECTS].identifier = mask_15;
             can_obj_buf[MAX_OBJECTS].obj_typ = D_REC;
             can_obj_buf[MAX_OBJECTS].standart = TRUE;
             can_obj_buf[MAX_OBJECTS].data_length = data_length;
             can_obj_buf[MAX_OBJECTS].task_bind = NO_BIND;
             can_obj_buf[MAX_OBJECTS].obj_read = TRUE;

             can_obj_buf[MAX_OBJECTS].contr_adr =
                 CONTR_BUF_START_ADR + (MAX_OBJECTS -1 ) * MESSAGE_SIZE ;

             c_init_obj (mask_15,
                        data_length,
                        D_REC,
                        can_obj_buf[MAX_OBJECTS].contr_adr,
                        FALSE ); /* 0 for extended */
             return C_OK;
          }
          else {
             /* Objekt existiert schon */
             return C_OBJ_ERROR;
          }
    }


#pragma eject
/*---------------------------------------------------------------------*
 *     Zugriffsfunktionen                                              *
 *---------------------------------------------------------------------*/

 /* Die Applikation benutzt die Objekt-Identifikation fr den Zugriff  */
 /* auf ein bestimmtes Objekt. Intern wird jedoch nur der Index eines  */
 /* bestimmten Objektes innerhalb des Objektbuffers zum Zugriff        */
 /* benutzt. Mit den Prozeduren C_GET_INDEX und C_GET_IDENT    */
 /* kann der Index resp. die Identifikation eines bestimmten Objektes  */
 /* ermittelt werden.                                                  */
 /*                                                                    */
 /* Die jeweiligen Zugriffsprozeduren testen aus Zeitgrnden jeweils   */
 /* nicht mehr ob ein bestimmter Index tatschlich einem definierten   */
 /* Objekt entspricht !                                                */


/*---------------------------------------------------------------------*
 *       C _ G E T _ I N D E X _ F R O M _ I D E N T
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Testet ob das Objekt IDENTIFIER im Objekt-Buffer definiert ist
 *     und gibt den entsprechenden Index innerhalb CAN_OBJ_BUF zurck.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     identifier   :    Identifikation des gesuchten Objektes
 *
 *     Returnwert   :    0        -->   Das Objekt IDENTIFIER existiert
 *                                      nicht
 *                       1 .. 14 -->   Index des Objektes IDENTIFIER
 *
 *---------------------------------------------------------------------*/
   byte c_get_index_from_ident (unsigned long identifier)

{
   byte i, temp;

   temp = nbr_of_obj;
   for (i=1; i<=temp; i++) {
      if (can_obj_buf[i].identifier == identifier) return i;
   }
   return 0;
}



/*---------------------------------------------------------------------*
 *     C _ G E T _ I D E N T
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Gibt den Identifier des Objektes mit dem INDEX innerhalb
 *     CAN_OBJ_BUF zurck.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     index        :    Index des gesuchten Objektes(1..14)
 *
 *     Returnwert   :    Identifikation des gewnschten Objektes
 *                       FFFFFFFFH      ->  Objekt existiert nicht
 *                                          oder 15. Objekt
 *---------------------------------------------------------------------*/
   unsigned long c_get_ident (byte index)

{
   if ((index == 0) || (index > nbr_of_obj)) return 0xffffffff;
   return can_obj_buf[index].identifier;
}


/*---------------------------------------------------------------------*
 *     C _ G E T _ R E C _ B I N D
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Gibt den Index eines neu empfangenen und noch nicht von der
 *     Applikation gelesenen Objektes, das fest an den Task TASK_ID
 *     gebunden ist, zurck.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     task_id   :    Task-Identifikation des Tasks an den das Objekt
 *                    gebunden ist
 *
 *     Returnwert   :    0        -->   Es existiert kein gewnschtes Objekt
 *                       1 .. 15  -->   Index des Objektes
 *
 *---------------------------------------------------------------------*/

byte c_get_rec_bind (byte task_id)
{
   byte i, ind;

   for (i=0; i<=MAX_BIND_OBJ-1; i++) {
      ind = bind_index_list[i];
      if (ind != 0) {
         if (can_obj_buf[ind].task_bind == task_id) {
            if (!(can_obj_buf[ind].obj_read)) {
               return ind;
            }
         }
      }
   }
   return 0;
}



/*---------------------------------------------------------------------*
 *     C _ G E T _ O B J _ T Y P
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Gibt den Objekttyp wie in C_DEFINE_OBJ definiert des Objektes
 *     INDEX zurck.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     index        :    Index des Objektes innerhalb CAN_OBJ_BUF dessen
 *                       Typ gelesen werden soll
 *
 *     Returnwert   :    Objekttyp des Objektes INDEX
 *
 *---------------------------------------------------------------------*/

byte c_get_obj_typ (byte index)
{
   return can_obj_buf[index].obj_typ;
}



/*---------------------------------------------------------------------*
 *     C _ W R I T E _ D A T A
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Schreiben von Daten aus dem Array ARR in das Objekt mit INDEX
 *     im Objekt-Buffer.
 *     Es werden soviele Datenbytes geschrieben wie DATA_LENGTH
 *     des entsprechenden Objektes angibt.
 *       Die Daten werden nur in das Objekt geschrieben, sie werden
 *     nicht gesendet.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     index      :      Index des Objektes innerhalb OBJ_BUF in das
 *                       geschrieben werden soll
 *
 *     arr        :      Zu schreibende Daten
 *
 *---------------------------------------------------------------------*/

void c_write_data (byte index,
                   byte xdata *arr)
{
   byte temp;

   if (can_obj_buf[index].data_length > 0) {
      /* Der Returnwert der Funktion wird nicht ausgewertet */
     /* Nur fr interne Testzwecke                           */
      temp = c_write_obj (can_obj_buf[index].contr_adr, arr);
   }
}



/*---------------------------------------------------------------------*
 *     C _ R E A D _ D A T A
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Lesen der Daten des Objektes INDEX im Objekt-Buffer. Es werden
 *     soviele Daten gelesen, wie DATA_LENGTH des entsprechenden
 *     Objektes angibt.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     index       :  Index des Objektes in OBJ_BUF aus dem gelesen
 *                    werden soll.
 *
 *     arr         :  Array in den die Daten gelesen werden sollen
 *
 *
 *---------------------------------------------------------------------*/

void c_read_data (byte index,
                  byte xdata *arr)
{
   byte length;

   length = can_obj_buf[index].data_length;
   if (length > 0) {
      c_read_obj (can_obj_buf[index].contr_adr, arr);
   }
   can_obj_buf[index].obj_read = TRUE;
}



/*---------------------------------------------------------------------*
 *     C _ R E C E I V E _ O B J
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Fhrt die nach einem Receive-Interrupt ntigen Funktionen aus.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *                      Normale Rckmeldung bei einwandfrei Empfangenen
 *                    Data-Frames :
 *                    1 bis 15   ->    Index innerhalb OBJ_BUF des
 *                                      empfangenen und abgespeicherten
 *                                        Objektes
 *
 *                                        Rckmeldungen falls kein
 *                                        Data-Frame fr die Applikation
 *                                      empfangen :
 *
 *                    FFF0H       ->    Ein Remote-Frame wurde empfangen und
 *                                      beantwortet
 *                    FFF1H       ->    Ein Remote-Frame wurde empfangen,
 *                                      das entsprechende Objekt hat aber
 *                                      einen falschen Typ
 *                    FFF2H       ->    Ein Remote-Frame wurde empfangen,
 *                                      die Antwort konnte aber wegen
 *                                      Bus-Problemen nicht gesendet werden
 *                    FFF3H       ->    Das Objekt hat die 2. Stufe der
 *                                      Akzeptanzfilterung nicht passiert
 *                    FFF4H       ->    Das Objekt hat die 3. Stufe der
 *                                      Akzeptanzfilterung nicht passiert
 *                    FFF5H       ->    Ein Objekt wurde empfangen und
 *                                      akzeptiert, das entsprechende
 *                                      Objekt hat aber einen falschen Typ
 *                    FFF6H       ->    Ein Objekt wurde empfangen,
 *                                      akzeptiert und abgespeichert,
 *                                      die Applikation muss aber nicht
 *                                      benachrichtigt werden, da das
 *                                      Objekt nur neu beschrieben wurde
 *                                      (seit dem letzten Empfang noch
 *                                      nicht gelesen durch die
 *                                      Applikation).
 *
 *
 *---------------------------------------------------------------------*/

unsigned int c_receive_obj (void)
{
      byte index, adr;

      adr = c_get_obj_int_adr();
      index = c_get_index_from_adr (adr);
      if (index == 0) {
         /* Objekt ist nicht bekannt */
         c_receipt_obj_int (adr);
         return 0xfff0;
      }

      /* Gltiges Data-Frame empfangen, Typ testen */
      if ((can_obj_buf[index].obj_typ == D_REC) ||
         (can_obj_buf[index].obj_typ == D_REC_R_SEND)) {
         /* Interrupt freigeben */
         c_receipt_obj_int (adr);
         if (can_obj_buf[index].obj_read) {
            can_obj_buf[index].obj_read = FALSE;
            return index;
         }
         else {
            return 0xfff2;
         }
      }
      else {
         /* falscher Objekttyp */
         c_receipt_obj_int (adr);
         return 0xfff1;
      }
}



/*---------------------------------------------------------------------*
 *     C _ T R A N S M I T _ O B J
 *---------------------------------------------------------------------*
 *  Funktion:
 *       Senden eines Data-Frames ber den CAN-Bus mit Abspeichern der
 *       Daten im Objektbuffer.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *       Index       :  Index des zu sendenden Objektes
 *
 *     arr         :  Array mit den Daten die gesendet werden sollen
 *
 *     Returnwert  :  TRUE  --> Objekt erfolgreich gesendet
 *                    FALSE --> Objekt konnte nicht gesendet werden
 *                              (Bus-Fehler)
 *
 *---------------------------------------------------------------------*/

byte c_transmit_obj (byte index,
                     byte xdata *arr)
{
   return c_send_obj (can_obj_buf[index].contr_adr, FALSE, arr);
}



/*---------------------------------------------------------------------*
 *     C _ S E N D _ R E M O T E
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Senden eines eines Remote-Frames ber den CAN-Bus
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     Index       :  Index des Objektes von dem ein Remote-Frame
 *                    gesendet werden soll
 *
 *       Returnwert  :  TRUE    --> Objekt erfolgreich gesendet
 *                    FALSE --> Objekt konnte nicht gesendet werden
 *                              (Bus-Fehler)
 *
 *---------------------------------------------------------------------*/

byte c_send_remote (byte index)
{
   return c_send_obj (can_obj_buf[index].contr_adr, TRUE, 0);
}



/*---------------------------------------------------------------------*
 *     C _ S E T _ B I N D
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Setzt das Feld TASK_BIND innerhalb des Objektes INDEX im
 *     Objekt-Buffer auf eine bestimmte TASK_ID (wird zum festen
 *     Anbinden eines Objektes an einen Task bentigt, der CAN-Driver
 *     kann damit ermitteln, zu welchem Task er ein Signal senden muss,
 *     wenn dieses Objekt empfangen wird). Gleichzeitig wird der Index
 *     des Objektes im Array BIND_INDEX_LIST abgespeichert.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     index      :  Index des Objekts bei dem der Task-Bind Wert
 *                   gesetzt werden soll
 *
 *     task_id    :  Wert auf den der Task-Bind Wert gesetzt werden soll
 *
 *     Returnwert  :  TRUE  -->   OK
 *                    FALSE -->   Die BIND_INDEX_LIST ist voll
 *                                (bereits 8 Objekte angebunden)
 *
 *---------------------------------------------------------------------*/

byte c_set_bind (byte index,
                 byte task_id)
{
   byte i;
   for (i=0; i<=MAX_BIND_OBJ-1; i++) {
      if ((bind_index_list[i] == 0) || (bind_index_list[i] == index)) {
         bind_index_list[i] = index;
         can_obj_buf[index].task_bind = task_id;
         return TRUE;
      }
   }
   return FALSE;
}



/*---------------------------------------------------------------------*
 *     C _ C L E A R _B I N D
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Markiert das Objekt als zu keinem bestimmten Task zugehrig.
 *     Setzt das Feld TASK_BIND innerhalb des Objektes INDEX im
 *     Objekt-Buffer auf FFH (in RTX-51 keine gltige TASK-ID).
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     index  : Index des Objekts bei dem der Task-Bind Wert gelscht
 *              werden soll
 *
 *---------------------------------------------------------------------*/

void c_clear_bind (byte index)
{
   byte i;

   for (i=0; i<=7; i++) {
      if (bind_index_list[i] == index) bind_index_list[i] = 0;
   }
   can_obj_buf[index].task_bind = NO_BIND;
}



/*---------------------------------------------------------------------*
 *     C _ G E T _ B I N D
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Gibt den aktuellen Wert des Feldes TASK_BIND im Objekt
 *     INDEX zurck. NO_BIND bedeutet dabei, dass das Objekt an keinen
 *     Task gebunden ist.
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     index       :  Index des Objekts dessen Task-Bind Wert gelesen
 *                    werden soll
 *
 *     Returnwert  :  Alle Werte ausser FFH sind eine gltige Task-ID.
 *                    FFH bedeutet, dass das Objekt an keinen Task
 *                    gebunden ist.
 *
 *---------------------------------------------------------------------*/

byte c_get_bind (byte index)
{
   return can_obj_buf[index].task_bind;
}



/*---------------------------------------------------------------------*
 *     C _ S E T _ R E A D
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Setzt OBJ_READ innerhalb des Objektes INDEX auf TRUE. Diese
 *     Funktion dient dazu ein neu empfangenes Objekt als von der
 *     Applikation gelesen zu markieren, falls die Applikation nicht
 *     getstartet werden kann (Mailbox voll usw.).
 *
 *---------------------------------------------------------------------*
 *  Parameter:
 *
 *     index       :  Index des Objekts bei dem OBJ_READ gesetzt
 *                    werden soll
 *
 *---------------------------------------------------------------------*/

void c_set_read (byte index)
{
   can_obj_buf[index].obj_read = TRUE;
}



/*---------------------------------------------------------------------*
 *     C _ S E T _ A L L _ R E A D
 *---------------------------------------------------------------------*
 *  Funktion:
 *     Setzt OBJ_READ innerhalb aller Objekte auf TRUE. Diese
 *     Funktion dient dazu alle empfangenen Objekte als von der
 *     Applikation gelesen zu markieren.
 *
 *---------------------------------------------------------------------*/

void c_set_all_read (void)
{
   byte i;

   for (i=1; i<=nbr_of_obj; i++) {
      can_obj_buf[i].obj_read = TRUE;
   }
}



/*---------------------------------------------------------------------*
 *     Ende Modul ICANOBJA                                             *
 *---------------------------------------------------------------------*/


